package cn.skcks.matrix.v2.services.location.record;

import cn.skcks.matrix.v2.model.location.record.convert.LocationRecordConvertor;
import cn.skcks.matrix.v2.model.location.record.dto.LocationRecordParams;
import cn.skcks.matrix.v2.model.location.record.dto.PageLocationRecordParams;
import cn.skcks.matrix.v2.model.location.record.vo.LocationRecordVo;
import cn.skcks.matrix.v2.model.services.ServiceResult;
import cn.skcks.matrix.v2.orm.mybatis.dynamic.mapper.LocationRecordDynamicSqlSupport;
import cn.skcks.matrix.v2.orm.mybatis.dynamic.mapper.LocationRecordMapper;
import cn.skcks.matrix.v2.orm.mybatis.dynamic.mapper.RoleDynamicSqlSupport;
import cn.skcks.matrix.v2.orm.mybatis.dynamic.model.LocationRecord;
import cn.skcks.matrix.v2.utils.page.PageWrapper;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.RequiredArgsConstructor;
import org.mybatis.dynamic.sql.SqlBuilder;
import org.springframework.stereotype.Service;

import java.math.BigInteger;
import java.util.Collection;
import java.util.List;

@Service
@RequiredArgsConstructor
public class LocationRecordServiceImpl implements LocationRecordService{
    private final LocationRecordMapper locationRecordMapper;
    @Override
    public ServiceResult<Collection<LocationRecord>> getLocationRecord(LocationRecordParams params, String userId) {
        List<LocationRecord> recordList = locationRecordMapper.select((s)-> s.applyWhere(dsl -> {
            if (params.getStartTime() != null) {
                dsl.and(LocationRecordDynamicSqlSupport.locationTime, SqlBuilder.isGreaterThanOrEqualTo(params.getStartTime()));
            }

            if (params.getEndTime() != null) {
                dsl.and(LocationRecordDynamicSqlSupport.locationTime,SqlBuilder.isLessThanOrEqualTo(params.getEndTime()));
            }

            dsl.and(LocationRecordDynamicSqlSupport.userId, SqlBuilder.isEqualTo(Long.valueOf(userId)));
            dsl.and(LocationRecordDynamicSqlSupport.id, SqlBuilder.isNotNull());
        }).orderBy((LocationRecordDynamicSqlSupport.id)));

        return ServiceResult.<Collection<LocationRecord>>builder()
                .result(recordList)
                .build();
    }

    @Override
    public ServiceResult<PageInfo<LocationRecord>> getPageLocationRecord(PageLocationRecordParams params, String userId) {
        PageInfo<LocationRecord> pageInfo;
        try(Page<LocationRecord> page = PageHelper.startPage(params.getPage(),params.getSize())){
            pageInfo = page.doSelectPageInfo(()->{
                getLocationRecord(params, userId);
            });
        }

        return ServiceResult.<PageInfo<LocationRecord>>builder()
                .result(pageInfo)
                .build();
    }
}
